import os
import json

from typing import Any

from ytrss.configuration.configuration import ConfigurationError, ConfigurationFileNotExistsError
from ytrss.configuration.readers.file_dict_configuration import FileDictConfiguration


class JsonConfigurationParseError(ConfigurationError):
    """ Json configuration error. """


class JsonConfigurationFileNotExistsError(ConfigurationFileNotExistsError):
    """ Json configuration file not exists """


class JsonConfiguration(FileDictConfiguration):
    """ Configuration json object """

    def read_data_from_file(self, conf_file: str) -> Any:
        try:
            with open(os.path.expanduser(conf_file), encoding="utf-8") as data_file:
                data = json.load(data_file)
        except FileNotFoundError as exc:
            raise JsonConfigurationFileNotExistsError() from exc

        if data == {}:
            raise JsonConfigurationParseError("Cannot find data from file")
        return data



from jinja2 import Environment, Template, FileSystemLoader

from ytrss import templates as ytrss_templates
from ytrss.core.helpers.templates import format_str, format_desc, format_date


class TemplatesManager:
    """ Templates Manager

    Manager that returns a template from ytrss package
    """

    def __init__(self) -> None:
        self._env = Environment(
            loader=FileSystemLoader(ytrss_templates.__path__[0]),
        )
        self._env.filters["format_str"] = format_str
        self._env.filters["format_desc"] = format_desc
        self._env.filters["format_date"] = format_date

    def _get_template(self, path: str) -> Template:
        return self._env.get_template(path)

    def __getitem__(self, key: str) -> Template:
        return self._get_template(key)



from typing import Optional


def first_line(text: Optional[str]) -> str:
    """
    Return first line of text
    """
    if text is not None:
        for line in text.split("\n"):
            if line != "":
                return line
    return ""



from argparse import Namespace

from ytrss.commands import BaseCommand


class GenerateCommand(BaseCommand):
    """
    Generate elements in destinations.
    """
    def __init__(self) -> None:
        BaseCommand.__init__(self, "generate")

    def run(self, options: Namespace) -> int:
        for destination in self.manager_service.destination_manager.destinations:
            destination.on_finish(self.manager_service.templates_manager)
        return 0



from typing import Optional

from ytrss.core.entity.destination import Destination

from ytrss.configuration.entity.destination_info import DestinationInfo

from ytrss.core.entity.plugin import Plugin
from ytrss.plugins.default.destination import DefaultDestination


class DefaultPlugin(Plugin):
    """ Plugin with default destination.

    All movies send to this destination will be only saved to selected directory.
    """

    def create_destination(self, info: DestinationInfo) -> Optional[Destination]:
        return DefaultDestination(info)



import os
from urllib.parse import urlparse

from mutagen import File as FileMutagen

from ytrss.configuration.entity.configuration_data import YtrssConfiguration
from ytrss.core.entity.downloaded_movie import DownloadedMovie
from ytrss.core.entity.plugin import Plugin
from ytrss.core.helpers.logging import logger


class Mp3TagsPlugin(Plugin):
    """ Mp3 tags plugin

    Plugin to add metadata to audio files.
    """

    def modify_res_files(self, downloaded_movie: DownloadedMovie, configuration: YtrssConfiguration) -> None:
        for res_file in downloaded_movie.resources_files:
            movie_tags = FileMutagen(os.path.join(configuration.cache_path, res_file), easy=True)

            if movie_tags is None:
                continue

            try:
                movie_tags["title"] = downloaded_movie.title
            except KeyError as key_error:
                logger.warning(key_error)

            try:
                movie_tags["artist"] = downloaded_movie.author
            except KeyError as key_error:
                logger.warning(key_error)

            try:
                movie_tags["album"] = urlparse(downloaded_movie.url).netloc
            except KeyError as key_error:
                logger.warning(key_error)

            try:
                movie_tags["comment"] = downloaded_movie.description
            except KeyError as key_error:
                logger.warning(key_error)

            try:
                movie_tags["date"] = downloaded_movie.date.strftime("%Y-%m-%d")
            except KeyError as key_error:
                logger.warning(key_error)

            movie_tags.save()



from typing import Optional, Callable, List

from ytrss.configuration.entity.source import Source
from ytrss.core.entity.plugin import Plugin
from ytrss.core.entity.source_downloader import SourceDownloader
from ytrss.plugins.youtube.youtube_channel_source_downloader import YouTubeChannelSourceDownloader
from ytrss.plugins.youtube.youtube_named_channel_source_downloader import YouTubeNamedChannelSourceDownloader
from ytrss.plugins.youtube.youtube_playlist_source_downloader import YouTubePlaylistSourceDownloader


class YouTubePlugin(Plugin):
    """ Plugin with sources for YouTube site """

    def create_source_downloader(self, source: Source) -> Optional[SourceDownloader]:
        sources_constructs: List[Callable[[Source], SourceDownloader]] = [
            YouTubePlaylistSourceDownloader,
            YouTubeChannelSourceDownloader,
            YouTubeNamedChannelSourceDownloader
        ]

        for source_downloader_class in sources_constructs:
            try:
                source_downloader = source_downloader_class(source)
                if source_downloader is not None:
                    return source_downloader
            except Exception:  # pylint: disable=W0703
                pass
        return None



class Version:
    """ Object represents version """

    def __init__(self) -> None:
        self._major = 0
        self._minor = 3
        self._patch = 5
        self._rc = 2

    @property
    def version(self) -> str:
        """ A string name of version """
        return f"{self._major}.{self._minor}.{self._patch}" + (f"rc{self._rc}" if self._rc > 0 else "")

    def __str__(self) -> str:
        return self.version



#!/usr/bin/env python3
# PYTHON_ARGCOMPLETE_OK
import logging
import sys
from argparse import ArgumentParser
from typing import Optional, Sequence, List

from ytrss.commands import BaseCommand
from ytrss.commands.configuration import ConfigurationCommand
from ytrss.commands.download import DownloadCommand
from ytrss.commands.generate import GenerateCommand
from ytrss.commands.run import RunCommand
from ytrss.commands.url import UrlCommand
from ytrss.commands.version import VersionCommand
from ytrss.configuration.configuration import ConfigurationFileNotExistsError, ConfigurationError
from ytrss.configuration.entity.configuration_data import YtrssConfiguration
from ytrss.configuration.factory import create_configuration
from ytrss.core.helpers.logging import DebugFormatter, ClientFormatter, logger
from ytrss.core.helpers.string_utils import first_line
from ytrss.core.managers.manager_service import default_manager_service

__subcommands__: List[BaseCommand] = [
    VersionCommand(),
    RunCommand(),
    GenerateCommand(),
    ConfigurationCommand(),
    GenerateCommand(),
    DownloadCommand(),
    UrlCommand()
]


def _option_args() -> ArgumentParser:
    parser = ArgumentParser(description="command line tool to manage ytrss package",
                            prog='ytrss')
    parser.add_argument("-c", "--conf", dest="config_file",
                        help="configuration file", default="", metavar="FILE")
    parser.add_argument("-d", "--debug-log", dest="log_debug",
                        action="store_true", default=False,
                        help="Enable debug logging")

    return parser


def _configure_logging(debug_log: bool) -> None:
    stream_handler = logging.StreamHandler()
    stream_handler.setFormatter(DebugFormatter() if debug_log else ClientFormatter())
    logging.basicConfig(
        level=logging.DEBUG if debug_log else logging.INFO,
        handlers=[stream_handler]
    )


def main(argv: Optional[Sequence[str]] = None) -> None:
    """
    Main function for command line program.
    """
    parser = _option_args()

    subparsers = parser.add_subparsers(title="commands", description="Use once of this commands", dest="command")
    subparsers.add_parser("help", help="Print help message")
    for command in __subcommands__:
        subparser = subparsers.add_parser(command.name, help=first_line(command.__doc__))
        command.arg_parser(subparser)

    options = parser.parse_args(argv)

    _configure_logging(options.log_debug)

    try:
        manager_service = default_manager_service()
        manager_service.configuration = YtrssConfiguration(create_configuration("ytrss", options.config_file))
    except ConfigurationFileNotExistsError:
        logger.warning("Configuration file not exists")
        sys.exit(1)
    except ConfigurationError as ex:
        logger.debug("%s: %s", type(ex), ex)
        logger.error("There are no configuration file")
        sys.exit(2)

    try:
        # pylint: disable=C0415
        import argcomplete
        argcomplete.autocomplete(parser)
    except ImportError:
        pass

    if options.command is None or options.command == "help":
        parser.print_help()
        sys.exit(0)

    for command in __subcommands__:
        if command.name == options.command:
            sys.exit(command(options))

    logger.error("command: %s", options.command)


if __name__ == "__main__":
    main()



"""
Testing Podcast module.
"""

import unittest

from ytrss.core.helpers.templates import format_desc


class TestPodcast(unittest.TestCase):  # pylint: disable=R0904
    """
    Podcast tests.
    """

    def test_description_file(self) -> None:
        """
        Check podcast description formatter
        """
        samples = [
            {
                "text": "test",
                "predict": "test",
            },
            {
                "text": "user@email.com",
                "predict": "<a href='mailto:user@email.com'>user@email.com</a>",
            },
            {
                "text": "http://test.com",
                "predict": "<a href='http://test.com'>http://test.com</a>",
            },
            {
                "text": "https://test.com",
                "predict": "<a href='https://test.com'>https://test.com</a>",
            },
            {
                "text": "httpd://test.com",
                "predict": "httpd://test.com",
            },
            {
                "text": "/redirect?one=one&two=two",
                "predict": "<a href='https://youtube.com/redirect?one=one&two=two'"
                           ">/redirect?one=one&two=two</a>",
            },
            {
                "text": "/redirect?one=one&q=https%3A%2F%2Ftest.com%2Ftest&two=two",
                "predict": "<a href='https://test.com/test'>https://test.com/test</a>",
            },
            {
                "text": "/redirect?one=one&two=two&q=https%3A%2F%2Ftest.com%2Ftest",
                "predict": "<a href='https://test.com/test'>https://test.com/test</a>",
            },
            {
                "text": "/redirect?one=one&two=two&q=https%3A%2F%2Ftest.com%2Ftest\n",
                "predict": "<a href='https://test.com/test'>https://test.com/test</a><br />\n",
            },
            {
                "text": "/redirect?q=https%3A%2F%2Ftest.com%2Ftest&one=one",
                "predict": "<a href='https://test.com/test'>https://test.com/test</a>",
            },
            {
                "text": "/redirect?q=https%3A%2F%2Ftest.com%2Ftest",
                "predict": "<a href='https://test.com/test'>https://test.com/test</a>",
            },
            {
                "text": "sth /redirect?q=https%3A%2F%2Ftest.com%2Ftest sth",
                "predict": "sth <a href='https://test.com/test'>https://test.com/test</a> sth",
            },
        ]
        for elem in samples:
            result = format_desc(elem['text'])
            self.assertEqual(result, elem['predict'])


if __name__ == "__main__":
    unittest.main()



from ytrss.core.helpers.logging import logger
from ytrss.core.managers.manager_service import ManagerService, default_manager_service


def prepare_urls(manager_service: ManagerService = default_manager_service()) -> bool:
    """ Find urls and save to database

    The algorithm search for movies in source and save to database when some new movies are searched.
    """
    logger.info("Search for new movies:")
    queue = manager_service.database
    for movie, destination in manager_service.sources_manager.movies:
        if queue.queue_mp3(movie, destination):
            logger.info("Found new movie: [%s] %s [%s] => %s",
                        movie.url,
                        movie.title,
                        movie.identity,
                        destination)
        else:
            logger.debug("Element exists: %s", movie.url)
    return True



import json
import os
import time
from datetime import datetime
from email import utils

from email.utils import parsedate_to_datetime
from typing import Any, Optional, Dict, Sequence

from ytrss.core.entity.movie import Movie
from ytrss.core.helpers.typing import Url, Path


class MovieFileError(Exception):
    """ Error in Downloaded movie object """


class MovieResourceFileError(MovieFileError):
    """ Movie resource file error

    Error throws when the resource file not exists
    """


class MovieJSONError(MovieFileError):
    """ Movie Json Error

    Error throws when the json description file is invalid
    """


class DownloadedMovie:
    """ Downloaded movie

    The object that represents downloaded movie in destination. It has a field with description
    of movie and list of additional files.
    """

    def __init__(self, destination_dir: Path, data: Dict[Any, str]) -> None:
        self.__date: Optional[datetime] = None
        self.__destination_dir = destination_dir
        self._data = data

        for resource_file in self.data_paths:
            if not os.path.isfile(resource_file):
                raise MovieResourceFileError

    @property
    def data_paths(self) -> Sequence[Path]:
        """ List of full paths of files according movie """
        result = [
            Path(os.path.join(self.__destination_dir,
                              file_path)) for file_path in self.resources_files
        ]
        result.append(Path(os.path.join(self.__destination_dir, f"{self.identity}.json")))
        return result

    @property
    def resources_files(self) -> Sequence[Path]:
        """ List of files according movie """
        if "resources" in self._data:
            return [Path(x) for x in self._data.get("resources", [])]
        # for backward compatibility, where the resources are not present
        return [Path(f"{self.identity}.mp3")]

    @property
    def date(self) -> datetime:
        """ create date """
        if self.__date is None:
            date_str = self._data.get("date", "")
            self.__date = parsedate_to_datetime(date_str)
        return self.__date

    @property
    def title(self) -> str:
        """ movie's title"""
        return self._data.get("title", "<no title>")

    @property
    def image(self) -> Optional[Url]:
        """ files image """
        image = self._data.get("image", "")
        return Url(image) if image != "" else None

    @property
    def url(self) -> Url:
        """ movie's url """
        return Url(self._data.get("url", ""))

    @property
    def author(self) -> str:
        """ author of downloaded movie """
        return self._data.get("uploader", "")

    @property
    def identity(self) -> str:
        """ movie's identity """
        return self._data.get("id", "")

    @property
    def description(self) -> str:
        """ movie's description """
        return self._data.get("description", "")

    @staticmethod
    def create_movie_desc(destination_path: Path, movie: Movie,
                          resource_list: Sequence[Path]) -> 'DownloadedMovie':
        """ Create DownloadedMovie from movie object and files. """
        metadata_path = os.path.join(destination_path, f"{movie.identity}.json")
        with open(metadata_path, 'w', encoding="utf-8") as file_handler:
            movie_data: Dict[str, Any] = {
                "id": movie.identity,
                "url": movie.url,
                "title": movie.title,
                "uploader": movie.author,
                "description": movie.description,
                "date": utils.formatdate(time.mktime(movie.date.timetuple()))
            }
            if movie.img_url is not None:
                movie_data["image"] = movie.img_url
            movie_data["resources"] = resource_list
            file_handler.write(json.dumps(movie_data, indent=4, sort_keys=True))
        return DownloadedMovie(destination_path, movie_data)

    @staticmethod
    def from_file(destination_dir: Path, filename_path: str) -> 'DownloadedMovie':
        """ Create DownloadedMovie from existed movie """
        json_file = os.path.join(destination_dir, filename_path)
        if not os.path.isfile(json_file):
            raise MovieJSONError
        with open(json_file, encoding="utf-8") as data_file:
            result = DownloadedMovie(destination_dir, json.load(data_file))
        return result



from typing import Any

from ytrss.configuration.readers.base import ConfigurationReader


class ConfigurationError(Exception):
    """ Configuration error """


class ConfigurationFileNotExistsError(ConfigurationError):
    """ Configuration file not exists error

    The error that raised when the configuration file is not existed.
    """


class Configuration:
    """ Configuration object """

    def __init__(self, reader: ConfigurationReader) -> None:
        self._reader = reader

    @property
    def conf(self) -> Any:
        """ Returns object with configuration data. """
        return self._reader.read_config



import logging

from typing import Dict

_GREY = "\x1b[38;20m"
_BLUE = "\x1b[38;5;39m"
_YELLOW = "\x1b[33;20m"
_RED = "\x1b[31;20m"
_BOLD_RED = "\x1b[31;1m"
_RESET = "\x1b[0m"


class BaseFormatter(logging.Formatter):
    """
    Default formatter
    """
    def __init__(self, message_schema: str) -> None:
        self._formats: Dict[int, str] = {
            logging.DEBUG: f"{_BLUE}{message_schema}{_RESET}",
            logging.INFO: f"{_RESET}{message_schema}{_RESET}",
            logging.WARNING: f"{_YELLOW}{message_schema}{_RESET}",
            logging.ERROR: f"{_RED}{message_schema}{_RESET}",
            logging.CRITICAL: f"{_BOLD_RED}{message_schema}{_RESET}",
        }
        super().__init__()

    def format(self, record: logging.LogRecord) -> str:
        log_fmt = self._formats.get(record.levelno)
        formatter = logging.Formatter(log_fmt)
        return formatter.format(record)


class DebugFormatter(BaseFormatter):
    """
    Logging formatter for logs in debug mode
    """

    def __init__(self) -> None:
        super().__init__("[%(name)s] %(levelname)s - %(message)s (\"%(pathname)s:%(lineno)d\")")


class ClientFormatter(BaseFormatter):
    """
    Logging formatter for logs in client app
    """

    def __init__(self) -> None:
        super().__init__("%(message)s")


logger = logging.getLogger("ytrss")



import io
import os

from ytrss.core.entity.destination import DestinationError
from ytrss.core.managers.templates_manager import TemplatesManager
from ytrss.plugins.default.destination import DefaultDestination

from ytrss.plugins.rss.podcast.podcast import Podcast

from ytrss.configuration.entity.destination_info import DestinationInfo
from ytrss.core.helpers.logging import logger


class RssDestination(DefaultDestination):
    """
    A destination that create Rss feed.
    """

    def __init__(self, info: DestinationInfo) -> None:
        # pylint: disable=C0123
        if type(self) == RssDestination and info.type != 'rss':
            raise DestinationError()
        DefaultDestination.__init__(self, info)

    def on_finish(self, templates_manager: TemplatesManager) -> None:
        if self.info.output_path is None:
            raise KeyError
        if os.path.isdir(self.info.output_path):
            logger.info("Generate RSS: %s", self.info.title)
            podcast = Podcast(self.info, templates_manager)
            for movie in self.saved_movies:
                logger.debug(" > %s", movie.title)
                try:
                    podcast.add_item(movie=movie)
                except ValueError:
                    logger.info("Cannot add item to rss [%s] %s", movie.identity, movie.title)
            podcast_file = os.path.join(self.info.output_path, "podcast.xml")
            logger.debug("Create rss file: \"%s\"", podcast_file)
            with io.open(podcast_file, "w", encoding="utf-8") as file_handler:
                file_handler.write(podcast.generate())



from typing import Optional

from ytrss.core.entity.destination import Destination

from ytrss.configuration.entity.destination_info import DestinationInfo

from ytrss.core.entity.plugin import Plugin
from ytrss.plugins.rss.destination import RssDestination


class RssPlugin(Plugin):
    """ Plugin with destination that creates rss channel. """

    def create_destination(self, info: DestinationInfo) -> Optional[Destination]:
        return RssDestination(info)



import os
from typing import Any
import yaml
from ytrss.configuration.configuration import ConfigurationError, ConfigurationFileNotExistsError
from ytrss.configuration.readers.file_dict_configuration import FileDictConfiguration


class YamlConfigurationParseError(ConfigurationError):
    """ Yaml configuration error """


class YamlConfigurationFileNotExistsError(ConfigurationFileNotExistsError):
    """ Yaml configuration file not exists error. """


class YamlConfiguration(FileDictConfiguration):
    """ Configuration yaml object """

    def read_data_from_file(self, conf_file: str) -> Any:
        try:
            with open(os.path.expanduser(conf_file), encoding="utf-8") as data_file:
                # pylint: disable=E1120
                data = yaml.safe_load(data_file)
        except FileNotFoundError as exc:
            raise YamlConfigurationFileNotExistsError(exc) from exc

        if data == {}:
            raise YamlConfigurationParseError("Cannot find data from file")
        return data



"""
Tools for downloading mp3 from YouTube subscription and playlists.

Installation
===========

There are a two method of installation C{ytrss} module.

From PyPI repository::

    pip install ytrss

From sources::

    git clone git@github.com:rafyco/ytrss.git
    cd ytrss
    python setup.py install

Usage
=====

To more information try to call:

    ytrss --help

"""

import sys

from ytrss.core.version import Version

__title__ = 'ytrss'
__version__ = Version().version
__url__ = 'https://github.com/rafyco/ytrss.git'
__author__ = 'Rafal Kobel'
__author_email__ = 'rafalkobel@rafyco.pl'
__license__ = 'GNU'

__required_python__ = (3, 6)

import ytrss.core.version


def check_python_version() -> None:
    """
    Check if python is in appropriate version
    """
    current_python = sys.version_info[:2]
    if current_python < __required_python__:
        current_python_str = '.'.join([str(el) for el in current_python])
        required_python_str = '.'.join([str(el) for el in __required_python__])
        sys.stderr.write(f"""
==========================
Unsupported Python version
==========================

This version of {__title__} requires Python {required_python_str}, but you're trying to
install it on Python {current_python_str}.
This may be because you are using a version of pip that doesn't
understand the python_requires classifier. Make sure you
have Python {required_python_str} or newer, then try again:

    $ python3 -m pip install --upgrade pip setuptools
    $ pip3 install {__title__}

""")
        sys.exit(1)


if __name__ != "__main__":
    check_python_version()



from typing import List, Iterator, Tuple

from ytrss.configuration.entity.destination_info import DestinationId
from ytrss.configuration.entity.source import Source
from ytrss.core.entity.movie import Movie
from ytrss.core.entity.source_downloader import SourceDownloader, SourceDownloaderError
from ytrss.core.helpers.logging import logger
from ytrss.core.managers.plugin_manager import PluginManager


class SourcesManager:
    """
    Sources manager

    An object that managed all sources.
    """
    def __init__(self, plugin_manager: PluginManager) -> None:
        self._sources: List[SourceDownloader] = []
        self._plugin_manager = plugin_manager

    def add_from_info(self, source: Source) -> None:
        """ Add source from info file. """
        try:
            self._sources.append(self._plugin_manager.create_source_downloader(source))
        except SourceDownloaderError:
            logger.error("Unknown url: %s", source.url)

    @property
    def movies(self) -> Iterator[Tuple[Movie, DestinationId]]:
        """ List new movies from all registered sources. """
        for source in self._sources:
            for movie, destination in source.movies:
                yield movie, destination



from argparse import ArgumentParser, Namespace

from ytrss.commands import BaseCommand
from ytrss.configuration.entity.destination_info import DestinationId
from ytrss.core.entity.movie import MovieError
from ytrss.core.helpers.logging import logger
from ytrss.core.helpers.typing import Url


class UrlCommand(BaseCommand):
    """
    Add url to download
    """
    def __init__(self) -> None:
        BaseCommand.__init__(self, "url")

    def arg_parser(self, parser: ArgumentParser) -> None:
        parser.add_argument("url",
                            help="Url that should be downloaded")
        parser.add_argument("-d", "--destination", dest="destination",
                            help="destination [default: default]", default="default", metavar="DEST")

    def run(self, options: Namespace) -> int:
        try:
            movie = self.manager_service.plugin_manager.create_movie(Url(options.url))
        except MovieError:
            logger.error("This is not valid url or movie not exists: %s", options.url)
            return 1

        destination = DestinationId(options.destination)
        if destination not in self.manager_service.destination_manager:
            logger.error("Destination [%s] is not defined in configuration", destination)
            return 1

        if self.manager_service.database.queue_mp3(movie, destination):
            logger.info("Movie [%s] added to queue", options.url)
        else:
            logger.error("Cannot add this url: %s to queue. It is probably downloaded", options.url)
            return 2
        return 0



import os
import shutil
from typing import Iterator, Sequence

from ytrss.core.entity.destination import Destination, DestinationError

from ytrss.core.entity.downloaded_movie import DownloadedMovie, MovieFileError

from ytrss.configuration.entity.destination_info import DestinationId, DestinationInfo
from ytrss.core.helpers.logging import logger
from ytrss.core.helpers.typing import Path
from ytrss.core.managers.templates_manager import TemplatesManager


class DefaultDestination(Destination):
    """
    A destination copy files to destination folder.
    """

    def __init__(self, info: DestinationInfo) -> None:
        # pylint: disable=C0123
        if type(self) == DefaultDestination and info.type != 'default' and info.type != 'copy':
            raise DestinationError()
        self.__info = info

    @property
    def identity(self) -> DestinationId:
        return self.__info.identity

    @property
    def info(self) -> DestinationInfo:
        return self.__info

    @property
    def saved_movies(self) -> Iterator[DownloadedMovie]:
        if self.info.output_path is None:
            logger.error("output_path is None")
            return
        for filename in os.listdir(self.info.output_path):
            if filename.endswith(".json"):
                try:
                    yield DownloadedMovie.from_file(self.info.output_path, filename)
                except MovieFileError as ex:
                    logger.info(ex)

    def save(self, files: Sequence[Path], templates_manager: TemplatesManager) -> None:
        if self.info.output_path is None:
            raise KeyError
        os.makedirs(self.info.output_path, exist_ok=True)
        for file in files:
            try:
                shutil.move(file, self.info.output_path)
            except shutil.Error as ex:
                logger.error(ex)
        self.on_finish(templates_manager)



import abc
import os

from typing import Any

from ytrss.configuration.configuration import ConfigurationError
from ytrss.configuration.readers.base import ConfigurationReader


class FileDictConfiguration(ConfigurationReader, metaclass=abc.ABCMeta):
    """ Configuration json object """

    @property
    def read_config(self) -> Any:
        return self.__configuration

    @abc.abstractmethod
    def read_data_from_file(self, conf_file: str) -> Any:
        """ Read data from file """

    def __init__(self, conf_file: str) -> None:
        if not os.path.isfile(os.path.expanduser(conf_file)):
            raise ConfigurationError("File not exists")

        self.__configuration = self.read_data_from_file(conf_file)



from typing import List

from ytrss.configuration.entity.destination_info import DestinationInfo
from ytrss.core.entity.downloaded_movie import DownloadedMovie
from ytrss.core.managers.templates_manager import TemplatesManager


class Podcast:
    """
    Class that generate rss file from DownloaderMovie
    """

    def __init__(self, podcast_info: DestinationInfo, templates_manager: TemplatesManager) -> None:
        self.__podcast_info = podcast_info
        self.__items: List[DownloadedMovie] = []
        self._templates_manager = templates_manager

    def add_item(self, movie: DownloadedMovie) -> None:
        """ Add downloaded movie to rss """
        self.__items.append(movie)

    def generate(self) -> str:
        """ Generate rss file for object """
        return self._templates_manager["podcast.xml"]\
            .render(podcast_info=self.__podcast_info, movies=self.__items)



from typing import Optional

from ytrss.configuration.entity.configuration_data import YtrssConfiguration
from ytrss.core.managers.destination_manager import DestinationManager
from ytrss.core.managers.plugin_manager import PluginManager
from ytrss.core.managers.sources_manager import SourcesManager
from ytrss.core.managers.templates_manager import TemplatesManager
from ytrss.database.database import Database
from ytrss.database.sqlite.sqlite_database import SqliteDatabase


class ManagerService:
    """ Manager service

    A container to singletons and constructors of many objects used in script
    """

    def __init__(self) -> None:
        self._configuration: Optional[YtrssConfiguration] = None
        self._sources_manager: Optional[SourcesManager] = None
        self._destination_manager: Optional[DestinationManager] = None
        self._plugin_manager: Optional[PluginManager] = None
        self._templates_manager: Optional[TemplatesManager] = None
        self._database: Optional[Database] = None

    @property
    def configuration(self) -> YtrssConfiguration:
        """ Ytrss configuration """
        if self._configuration is None:
            raise ValueError("Configuration not exists")
        return self._configuration

    @configuration.setter
    def configuration(self, configuration: YtrssConfiguration) -> None:
        """ Ytrss configuration setter """
        self._configuration = configuration

    @property
    def sources_manager(self) -> SourcesManager:
        """ Sources manager """
        if self._sources_manager is None:
            self._sources_manager = SourcesManager(self.plugin_manager)
            for sources in self.configuration.sources:
                self._sources_manager.add_from_info(sources)
        return self._sources_manager

    @property
    def destination_manager(self) -> DestinationManager:
        """ Destination manager """
        if self._destination_manager is None:
            self._destination_manager = DestinationManager(self.plugin_manager)
            for _, info_value in self.configuration.destinations.items():
                self._destination_manager.add_from_info(info_value)
        return self._destination_manager

    @property
    def plugin_manager(self) -> PluginManager:
        """ Plugin manager """
        if self._plugin_manager is None:
            self._plugin_manager = PluginManager()
        return self._plugin_manager

    @property
    def templates_manager(self) -> TemplatesManager:
        """ Templates manager """
        if self._templates_manager is None:
            self._templates_manager = TemplatesManager()
        return self._templates_manager

    @property
    def database(self) -> Database:
        """
        Build defined object from parameter
        """
        if self._database is None:
            self._database = SqliteDatabase(self.configuration, self.plugin_manager)
        return self._database


_manager_service = ManagerService()


def default_manager_service() -> ManagerService:
    """ Return default Manager Service """
    return _manager_service



import os
from argparse import Namespace

from locks import Mutex

from ytrss.commands import BaseCommand
from ytrss.core.algoritms.download import download_all_movies
from ytrss.core.algoritms.finder import prepare_urls
from ytrss.core.helpers.logging import logger


class RunCommand(BaseCommand):
    """
    Find movie from sources and download
    """
    def __init__(self) -> None:
        BaseCommand.__init__(self, "run")

    def run(self, options: Namespace) -> int:
        os.makedirs('/tmp/ytrss', exist_ok=True)
        try:
            with Mutex('/tmp/ytrss/ytrss.lock'):
                prepare_urls()
                download_all_movies()
        except BlockingIOError:
            logger.info("Program is already started")
            return 1
        return 0



from argparse import Namespace
from youtube_dl.version import __version__ as youtube_dl_version

from ytrss.core.version import Version
from ytrss.commands import BaseCommand
from ytrss.core.helpers.logging import logger


class VersionCommand(BaseCommand):
    """
    Show version of this program
    """

    def __init__(self) -> None:
        BaseCommand.__init__(self, "version")

    def run(self, options: Namespace) -> int:
        logger.info("ytrss: %s", Version().version)
        logger.info("youtube_dl: %s", youtube_dl_version)
        return 0



import os
from typing import Sequence, Union

import youtube_dl

from ytrss.configuration.entity.configuration_data import YtrssConfiguration
from ytrss.core.entity.downloaded_movie import DownloadedMovie
from ytrss.core.entity.movie import Movie
from ytrss.core.helpers.exceptions import DownloadMovieError

from ytrss.core.helpers.files import cwd
from ytrss.core.helpers.logging import logger
from ytrss.core.helpers.typing import Path
from ytrss.plugins.youtube_dl.movie import YouTubeMovie


class YouTubeDownloader:
    """
    YouTube downloader

    Downloader that use `youtube_dl <https://rg3.github.io/youtube-dl/>`_ implementation. It downloads
    a movie file from any url that can be use by `youtube_dl` service.
    """

    def __init__(self, movie: Movie, configuration: YtrssConfiguration) -> None:
        if not isinstance(movie, YouTubeMovie):
            raise DownloadMovieError()
        self.configuration = configuration
        self._movie = movie

    @classmethod
    def __invoke_ytdl(cls, args: Sequence[str]) -> int:
        try:
            youtube_dl.main(args)
            status: Union[str, int] = 0
        except SystemExit as ex:
            if ex.code is None:
                status = 0
            else:
                status = ex.code  # pylint: disable=E0012,R0204
        if isinstance(status, str):
            return 0
        return status

    def download(self) -> DownloadedMovie:
        """ Download movie using YouTube_dl tool """
        try:
            os.makedirs(self.configuration.cache_path)
        except OSError:
            pass

        with cwd(self.configuration.cache_path):

            logger.info("Downloading movie: [%s] %s", self._movie.url, self._movie.title)
            status = self.__invoke_ytdl(list(self.configuration.args) + ['-o', f"{self._movie.identity}.mp3",
                                                                         self._movie.url])

            if status != 0:
                raise DownloadMovieError(f"youtube_dl raise with state: {status}")

            full_file_name = Path(f"{self._movie.identity}.mp3")
            if not os.path.isfile(full_file_name):
                raise DownloadMovieError(f"File {full_file_name} not downloaded")

        return DownloadedMovie.create_movie_desc(self.configuration.cache_path, self._movie, [full_file_name])
